@@ -1,7 +1,7 @@
 # Copyright (C) 2020 Juergen Nickelsen <ni@w21.org>, see LICENSE.
 
 """POSIX-conformant command-line option parser (plus long options)
-See the parse() function for details. Build 2020.809.1501
+See the parse() function for details. Build 2022.812.1632
 """
 
 import os
@@ -14,7 +14,10 @@ ErrorArg = "option does not take an argument"      # option
 ErrorNoarg = "option needs argument"               # option
 ErrorIntarg = "option argument must be integer"    # option
 ErrorMinarg = "too few arguments, needs at least"  # minimum
-ErrorMaxarg = "too many arguments, at most"        # maximum
+ErrorMaxarg = "too many arguments, takes at most"  # maximum
+
+class ArgumentError(Exception):
+    pass
 
 class OptionError(Exception):
     pass
@@ -32,9 +35,9 @@ class OptionValueContainer:
         for opt, desc in self._opts.items():
             if opt.startswith("_"):
                 assert opt in _keywords, "keyword unknown: " + repr(opt)
-            # While the following is grossly redundant, it is a kludge to
-            # finally reach 100% test coverage; apparently unpossible if both
-            # statements are under the same "if". Good luck we don't need to
+            # The following is grossly redundant -- it is a kludge to finally
+            # reach 100% test coverage; apparently impossible if both
+            # statements are under the same "if". Luckly we don't need to
             # optimise for speed. :-(
             if opt.startswith("_"):
                 continue
@@ -91,9 +94,9 @@ class OptionValueContainer:
                 while arg:
                     arg = self._have_opt(arg[0], arg[1:])
         if self._min is not None and len(self._args) < self._min:
-            raise OptionError(ErrorMinarg, self._min)
+            raise ArgumentError(ErrorMinarg, self._min)
         if self._max is not None and len(self._args) > self._max:
-                raise OptionError(ErrorMaxarg, self._max)
+                raise ArgumentError(ErrorMaxarg, self._max)
 
 
     def _have_opt(self, opt, arg=None):
@@ -149,6 +152,7 @@ class OptionValueContainer:
         msg = self.ovc_usage_msg() + "\n"
         if self._help_header:
             msg += self._help_header + "\n\n"
+        optparts = []
         for opt in sorted(self._opts.keys()):
             if opt.startswith("_"):
                 continue
@@ -156,8 +160,13 @@ class OptionValueContainer:
             arg = ""
             if desc[1] in (str, int):
                 arg = " " + (desc[4] if len(desc) == 5 else "ARG")
-            msg += " -%s, --%s%s:\n    %s" % (
-                opt, desc[0].replace('_', '-'), arg, desc[3])
+            optparts.append(f" -{opt}, --{desc[0].replace('_', '-')}{arg}:")
+        maxoptpartlen = max(map(len, optparts))
+        for opt in sorted(self._opts.keys()):
+            if opt.startswith("_"):
+                continue
+            desc = self._opts[opt]
+            msg += f"{optparts.pop(0):{maxoptpartlen}} {desc[3]}"
             if desc[1] in (int, str):
                 msg += " (%s arg, default %s)" % (
                     desc[1].__name__, repr(desc[2]))
@@ -233,80 +242,94 @@ def parse(descriptors, args=sys.argv[1:], exit_on_error=True):
 
     A key may also be one of these keywords:
 
-      "_arguments": string to print in the usage to describe the
-      non-option arguments, or, for argument count checking, a sequence
-      with the argument names:
-    
-         - a normal string counts as one argument towards minimum and
-           maximum
+    `_arguments`: string to print in the usage to describe the
+    non-option arguments, or, for argument count checking, a sequence
+    with the argument names:
 
-         - if it contains '...', there is no maximum the number of
-           arguments
+     - a normal string counts as one argument towards minimum and
+       maximum
 
-         - if it begins with '[', it is optional; if it can be split by
-           blanks into multiple words, each one counts toward the
-           maximum; e.g. "[param1 param2 param3]" increases the maximum
-           by 3, but not the minimum
+     - if it contains '...', there is no maximum number of arguments
 
-      "_help_footer": string to print with 'help' after the option
-      explanations
+     - if it begins with '[', it is optional; if it can be split by
+       blanks into multiple words, each one counts toward the
+       maximum; e.g. "[param1 param2 param3]" increases the maximum
+       by 3, but not the minimum
 
-      "_help_header": string to print with 'help' before the option
-      explanations
+    `_help_footer`: string to print with 'help' after the option
+    explanations
 
-      "_program": string to use as program name for help and usage
-      message instead of sys.argv[0]
+    `_help_header`: string to print with 'help' before the option
+    explanations
 
-      "_usage": string to use as usage message instead of the default
-      constructed one
+    `_program`: string to use as program name for help and usage
+    message instead of sys.argv[0]
 
-    If no '?' or 'h' option is specified, they will default to a long
-    form of '--help' and a 'help' function, which will be called
+    `_usage`: string to use as usage message instead of the default
+    constructed one
+
+    If no `-?` or `-h` option is specified, they will default to a long
+    form of `--help` and a `help` function, which will be called
     immediately when the option is seen. It prints a brief summary of
     the program's parameters and a description of the options, framed
-    by the _help_header and the _help_footer; it terminates the program
+    by the `_help_header` and the `_help_footer`; it terminates the program
     after printing the message.
 
     In case of a normal return of parse() (i.e. options and number of
-    arguments okay), it returns an OptionValueContainer and a list of
+    arguments okay), it returns an `OptionValueContainer` and a list of
     the remaining command line arguments. Example:
 
-      ovc, args = pgetopt.parse({
-      # opt: (name,          type, default value, helptext[, arg name])
-        "s": ("schmooze",    bool, 0,    "more schmooziness"),
-        "o": ("output_file", str,  None, "output file (or stdout)", "NAME"),
-        "n": ("repetitions", int,  3,    "number of repetitions"),
-        "d": ("debug",       str, [],    "debug topics", "DEBUG_TOPIC"),
-      # keyword:        value
-        "_arguments":   ["string_to_print", "..."],
-        "_help_header": "print a string a number of times",
-        "_help_footer": "This is just an example program.",
-      }
+        ovc, args = pgetopt.parse({
+        # opt: (name,          type, default value, helptext[, arg name])
+          "s": ("schmooze",    bool, 0,    "more schmooziness"),
+          "o": ("output_file", str,  None, "output file (or stdout)", "NAME"),
+          "n": ("repetitions", int,  3,    "number of repetitions"),
+          "d": ("debug",       str, [],    "debug topics", "DEBUG_TOPIC"),
+        # keyword:        value
+          "_arguments":   ["string_to_print", "..."],
+          "_help_header": "print a string a number of times",
+          "_help_footer": "This is just an example program.",
+        }
 
-      On return, ovc has the following fields:
-        ovc.schmooze:    number of -s options counted,
-        ovc.output_file: parameter of -o or --output-file, or None
-        ovc.repetitions: parameter of -n or --repetitions, or 3
-        ovc.debug:       list with all parameters given to -d or --debug
+    On return, `ovc` has the following fields:
 
-    Parameters to int or str options are taken from the next argument;
-    with long options, "--option=parameter" is also possible.
+      `ovc.schmooze`:    number of `-s` options counted,
+
+      `ovc.output_file`: parameter of `-o` or `--output-file`, or `None`
+
+      `ovc.repetitions`: parameter of `-n` or `--repetitions`, or `3`
+
+      `ovc.debug`:       list with all parameters given to `-d` or `--debug`
+
+    Parameters to `int` or `str` options are taken from the next argument;
+    with long options, `--option=parameter` is also possible.
 
     Other potentially useful fields of ovc:
-      ovc.ovc_help():  help function
-      ovc.ovc_usage(): usage function
+
+      `ovc.ovc_help()`:  help function
+
+      `ovc.ovc_usage()`: usage function
     
-      ovc.ovc_help_msg(),
-      ovc.ovc_usage_msg(): get corresponding messages as strings
+      `ovc.ovc_help_msg(),
+      ovc.ovc_usage_msg()`: get corresponding messages as strings
 
     """
     ovc = OptionValueContainer(descriptors, args)
+    exception = None
     try:
         ovc._parse()
         return ovc, ovc._args
     except OptionError as e:
-        if exit_on_error:
-            ovc.ovc_usage(e.args[0] + ": " + repr(e.args[1]))
-        raise(e)
+        exception = e
+        msg, opt = e.args
+        dash = "-" if len(opt) == 1 else "--"
+        message = f"{msg}: {dash}{opt}"
+    except ArgumentError as e:
+        exception = e
+        message = f"{e.args[0]} {e.args[1]}"
+    if exit_on_error:
+        ovc.ovc_usage(message)
+    else:
+        raise(exception)
 
 # EOF
